#========================================================================================
# package      DE_EPAGES::Order::t::unit::PaymentMethodFilter
# state        public
#----------------------------------------------------------------------------------------
# description  unit test for API/PaymentMethodFilter.pm
#========================================================================================
package DE_EPAGES::Order::t::unit::PaymentMethodFilter;
use base DE_EPAGES::Core::API::Test;
use Test::More;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::Mockery;
use DE_EPAGES::Test::Mock::Mockify qw ( GetParametersFromMockifyCall );
use DE_EPAGES::Order::API::Constants qw ( CLASSICCHECKOUT MULTISTEPCHECKOUT );
use DE_EPAGES::Test::API::TestTools qw ( TestExistsMethod );
use DE_EPAGES::Test::Mock::MockifyServlet;
use strict;

sub test {
    my $self = shift;

    $self->test_classicCheckout_possible();
    $self->test_classicCheckout_notPossible();
    $self->test_multiStepCheckout_possible();
    $self->test_multiStepCheckout_notPossible();

    $self->test_classicCheckout_mobileCheckout_possible();
    $self->test_classicCheckout_mobileCheckout_notPossible();
    $self->test_multiStepCheckout_mobileCheckout_possible();
    $self->test_multiStepCheckout_mobileCheckout_notPossible();

    $self->test_request_from_RestApi();

    $self->test_classicCheckout_undefAppServer_possible();
    $self->test_multiStepCheckout_undefAppServer_possible();

    done_testing();
}
#----------------------------------------------------------------------------------------
sub test_classicCheckout_possible {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $Mockery = $self->_createMocks(
        {
            'OrderProcess' => CLASSICCHECKOUT,
            'IsMobile' => 0,
        }
    );
    my $PaymentMethod = $Mockery->get('PaymentMethodCanDoCheckout');
    my $Result = $Mockery->get('SUT')->canDoCheckoutType($PaymentMethod);
    is($Result, 1, "$SubTestName - tests if the payment method can do classiccheckout.");
}
#----------------------------------------------------------------------------------------
sub test_classicCheckout_notPossible {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $Mockery = $self->_createMocks(
        {
            'OrderProcess' => CLASSICCHECKOUT,
            'IsMobile' => 0,
        }
    );
    my $PaymentMethod = $Mockery->get('PaymentMethodCanNotDoCheckout');
    my $Result = $Mockery->get('SUT')->canDoCheckoutType($PaymentMethod);
    is($Result, 0, "$SubTestName - tests if the payment method can not do classiccheckout.");
}
#----------------------------------------------------------------------------------------
sub test_multiStepCheckout_possible {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $Mockery = $self->_createMocks(
        {
            'OrderProcess' => MULTISTEPCHECKOUT,
            'IsMobile' => 0,
        }
    );
    my $PaymentMethod = $Mockery->get('PaymentMethodCanDoCheckout');
    my $Result = $Mockery->get('SUT')->canDoCheckoutType($PaymentMethod);
    is($Result, 1, "$SubTestName - tests if the payment method can do multistepcheckout.");
}
#----------------------------------------------------------------------------------------
sub test_multiStepCheckout_notPossible {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $Mockery = $self->_createMocks(
        {
            'OrderProcess' => MULTISTEPCHECKOUT,
            'IsMobile' => 0,
        }
    );
    my $PaymentMethod = $Mockery->get('PaymentMethodCanNotDoCheckout');
    my $Result = $Mockery->get('SUT')->canDoCheckoutType($PaymentMethod);
    is($Result, 0, "$SubTestName - tests if the payment method can not do multistepcheckout.");
}
#----------------------------------------------------------------------------------------
sub test_multiStepCheckout_mobileCheckout_possible {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $Mockery = $self->_createMocks(
        {
            'OrderProcess' => MULTISTEPCHECKOUT,
            'IsMobile' => 1,
        }
    );
    my $PaymentMethod = $Mockery->get('PaymentMethodCanDoCheckout');
    my $Result = $Mockery->get('SUT')->canDoCheckoutType($PaymentMethod);
    is($Result, 1, "$SubTestName - tests if the payment method can do mobilecheckout when multistepcheckout is the activated orderprocess.");
}
#----------------------------------------------------------------------------------------
sub test_multiStepCheckout_mobileCheckout_notPossible {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $Mockery = $self->_createMocks(
        {
            'OrderProcess' => MULTISTEPCHECKOUT,
            'IsMobile' => 1,
        }
    );
    my $PaymentMethod = $Mockery->get('PaymentMethodCanNotDoCheckout');
    my $Result = $Mockery->get('SUT')->canDoCheckoutType($PaymentMethod);
    is($Result, 0, "$SubTestName - tests if the payment method can not do mobilecheckout when multistepcheckout is the activated orderprocess.");
}
#----------------------------------------------------------------------------------------
sub test_classicCheckout_mobileCheckout_possible {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $Mockery = $self->_createMocks(
        {
            'OrderProcess' => CLASSICCHECKOUT,
            'IsMobile' => 1,
        }
    );
    my $PaymentMethod = $Mockery->get('PaymentMethodCanDoCheckout');
    my $Result = $Mockery->get('SUT')->canDoCheckoutType($PaymentMethod);
    is($Result, 1, "$SubTestName - tests if the payment method can do mobilecheckout when classiccheckout is the activated orderprocess.");
}
#----------------------------------------------------------------------------------------
sub test_classicCheckout_mobileCheckout_notPossible {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $Mockery = $self->_createMocks(
        {
            'OrderProcess' => CLASSICCHECKOUT,
            'IsMobile' => 1,
        }
    );
    my $PaymentMethod = $Mockery->get('PaymentMethodCanNotDoCheckout');
    my $Result = $Mockery->get('SUT')->canDoCheckoutType($PaymentMethod);
    is($Result, 0, "$SubTestName - tests if the payment method can not do mobilecheckout when classiccheckout is the activated orderprocess.");
}
#----------------------------------------------------------------------------------------
sub test_request_from_RestApi {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createMocks(
        {
            'OrderProcess' => CLASSICCHECKOUT,
            'IsMobile' => 1,
            'UseRestServlet' => 1,
        }
    );
    my $SUT = $Mockery->get('SUT');
    my $PaymentMethod = $Mockery->get('PaymentMethodCanDoCheckout');
    $SUT->canDoCheckoutType($PaymentMethod);

    my $aParams = GetParametersFromMockifyCall($SUT, '_canPaymentMethodCheckoutType');
    my $CheckoutType = $aParams->[1];

    is($CheckoutType, 'ClassicCheckout', "$SubTestName - check if REST API request never leads to MobileCheckout");
}

#----------------------------------------------------------------------------------------
sub test_classicCheckout_undefAppServer_possible {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $Mockery = $self->_createMocks(
        {
            'OrderProcess' => CLASSICCHECKOUT,
            'IsAppServerUndef' => 1,
        }
    );
    my $PaymentMethod = $Mockery->get('PaymentMethodCanDoCheckout');
    my $Result = $Mockery->get('SUT')->canDoCheckoutType($PaymentMethod);
    is($Result, 1, "$SubTestName - tests if classiccheckout is possible when there is no AppServer to verify if current servlet uses mobile sf.");
}

#----------------------------------------------------------------------------------------
sub test_multiStepCheckout_undefAppServer_possible {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $Mockery = $self->_createMocks(
        {
            'OrderProcess' => MULTISTEPCHECKOUT,
            'IsAppServerUndef' => 1,
        }
    );
    my $PaymentMethod = $Mockery->get('PaymentMethodCanDoCheckout');
    my $Result = $Mockery->get('SUT')->canDoCheckoutType($PaymentMethod);
    is($Result, 1, "$SubTestName - tests if multistepcheckout is possible when there is no AppServer to verify if current servlet uses mobile sf.");
}

#---------------------------------------------------------------------------
#  helpers
#---------------------------------------------------------------------------
sub _createMocks {
    my $self = shift;
    my ($hSetup) = @_;
    my $Mockery = DE_EPAGES::Test::Mock::Mockery->new();

    $Mockery->extend($self->_createAppServer($hSetup));

    $Mockery->register('SUT', ['AppServer'], sub {
        my ($AppServer) = $hSetup->{'IsAppServerUndef'} ? undef : @_;
        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Order::API::PaymentMethodFilter', []);
        $Mockify->mock('_GetAppServer', $AppServer);
        $Mockify->addMethodSpyWithParameterCheck('_canPaymentMethodCheckoutType', [{'object' => 'DE_EPAGES::Payment::API::Object::PaymentMethod'}, 'string']);
        return $Mockify->getMockObject();
    });

    $Mockery->register('Shop', [], sub{
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Shop::API::Object::Shop',
            {
                'OrderProcess' => $hSetup->{'OrderProcess'}
            }
        );
        return $Mockify->getMockObject();
    });

    $Mockery->register('PaymentMethodCanDoCheckout', ['PaymentTypeCanDoCheckout', 'Shop'], sub{
        my ($PaymentTypeCanDoCheckout, $Shop) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Payment::API::Object::PaymentMethod',
            {
                'PaymentType' => $PaymentTypeCanDoCheckout,
                'Shop' => $Shop,
                'instanceOf' => 'PaymentMethod',
            }
        );
        return $Mockify->getMockObject();
    });

    $Mockery->register('PaymentMethodCanNotDoCheckout', ['PaymentTypeCanNotDoCheckout', 'Shop'], sub{
        my ($PaymentTypeCanNotDoCheckout, $Shop) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Payment::API::Object::PaymentMethod',
            {
                'PaymentType' => $PaymentTypeCanNotDoCheckout,
                'Shop' => $Shop,
                'instanceOf' => 'PaymentMethod',
            }
        );
        return $Mockify->getMockObject();
    });

    $Mockery->register('PaymentTypeCanDoCheckout', [], sub{
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Payment::API::Object::PaymentType',
            {
                'CanDoMobileCheckout' => 1,
                'CanDoClassicCheckout' => 1,
                'CanDoMultiStepCheckout' => 1,
            }
        );
        return $Mockify->getMockObject();
    });

    $Mockery->register('PaymentTypeCanNotDoCheckout', [], sub{
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Payment::API::Object::PaymentType',
            {
                'CanDoMobileCheckout' => 0,
                'CanDoClassicCheckout' => 0,
                'CanDoMultiStepCheckout' => 0,
            }
        );
        return $Mockify->getMockObject();
    });
    return $Mockery->assemble();
}
#----------------------------------------------------------------------------------------
sub _createAppServer {
    my $self = shift;
    my ($hSetup) = @_;

    my $UseRestServlet = $hSetup->{'UseRestServlet'};
    my $ServletName = $UseRestServlet ? 'RestServlet' : 'Servlet';

    my $Mockery = DE_EPAGES::Test::Mock::Mockery->new();
        $Mockery->register('AppServer', [$ServletName], sub {
        my ($Servlet) = @_;
        my $MockObject = Test::MockObject->new();
        TestExistsMethod('DE_EPAGES::WebInterface::API::ApplicationServer','currentServlet');
        $MockObject->mock('currentServlet',sub {
            return $Servlet;
        });
        return $MockObject;
    });

    $Mockery->register('RestServlet', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new( 'DE_EPAGES::WebService::Servlet::Rest', []);
        $Mockify->getMockObject();
    });

    $Mockery->register('Servlet', ['BaseObject'], sub {
        my ($BussinessObject) = @_;
        my $MockifyServlet = DE_EPAGES::Test::Mock::MockifyServlet->new($BussinessObject);
        $MockifyServlet->addVar( 'IsMobile', $hSetup->{'IsMobile'} );
        $MockifyServlet->getMockObject();
    });

    $Mockery->register('BaseObject', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new( 'DE_EPAGES::Core::API::Object::BaseObject', { } );
        return $Mockify->getMockObject();
    });

    return $Mockery->assemble();
}
__PACKAGE__->runScript();