#========================================================================================
# package      DE_EPAGES::PayPalPlus::t::unit::UI_Shop
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  unit test for UI::Shop
#========================================================================================
package DE_EPAGES::PayPalPlus::t::unit::UI_Shop;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::Mockify qw ( WasCalled GetParametersFromMockifyCall );
use DE_EPAGES::Test::Mock::MockifyServlet;
use Test::MockObject::Extends;
use Test::Exception;
use Test::More;

use DE_EPAGES::PayPalPlus::UI::Shop;

use strict;

sub test {
    my $self = shift;
    $self->{'__setup'} = {};
    $self->{'__toCheck'} = {};
    $self->test_ViewPayPalPlusCancelPayment();
    $self->test_ViewPayPalPlusCancelPayment_WithInvalidGuid();
    $self->test_ViewPayPalPlusCancelPayment_UnityShop();

    $self->test_PaymentBacklinkPayPalPlus();
    $self->test_PaymentBacklinkPayPalPlus_FailToProcessOrder();
    $self->test_PaymentBacklinkPayPalPlus_FailToProcessOrder_UnityShop();

    done_testing();
}

sub test_ViewPayPalPlusCancelPayment {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    $self->{'__setup'}->{'IsUnityShop'} = 0;

    my $WasOrderLoaded = 1;
    my $WasDoRequestSuccessful = 1;
    my $ProccessOrderId = 42;
    my $UIShop = $self->_createUIShop($WasOrderLoaded, $WasDoRequestSuccessful, $ProccessOrderId);
    my $Servlet = $self->_buildServlet();

    lives_ok(sub { $UIShop->ViewPayPalPlusCancelPayment($Servlet) }, "$SubTestName - Test if ViewPayPalPlusCancelPayment lives");
    my $aParamsExecuteFormError = GetParametersFromMockifyCall($Servlet->form(), 'executeFormError', 0);

    my $hParams = $aParamsExecuteFormError->[0];
    is($hParams->{'ViewAction'}, 'ViewPaymentPayPalPlusResult', "$SubTestName - Test if ViewAction is correct");
    is($hParams->{'Reason'}, 'PayPalPlus_PaymentFailed', "$SubTestName - Test if Reason is cancel");
    is($hParams->{'Name'}, 'PayPalPlus_PaymentFailed', "$SubTestName - Test if Name is cancel");
    is($hParams->{'ObjectID'}, 'CustomerOrderID', "$SubTestName - Test if ObjectId is right");
    is($hParams->{'ObjectClass'}, 'CustomerOrder',"$SubTestName - Test if ObjectClass is Order");
}

sub test_ViewPayPalPlusCancelPayment_WithInvalidGuid {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    $self->{'__setup'}->{'IsUnityShop'} = 0;

    my $WasOrderLoaded = 0;
    my $WasDoRequestSuccessful = 1;
    my $ProccessOrderId = 0;
    my $UIShop = $self->_createUIShop($WasOrderLoaded, $WasDoRequestSuccessful, $ProccessOrderId);
    my $Servlet = $self->_buildServlet();

    lives_ok(sub { $UIShop->ViewPayPalPlusCancelPayment($Servlet); }, "$SubTestName - Test if ViewPayPalPlusCancelPayment lives");

    my $Form = $self->{'__toCheck'}->{'Form'};
    my $aParamsExecuteFormError = GetParametersFromMockifyCall($Form, 'executeFormError', 0);
    my $hParams = $aParamsExecuteFormError->[0];
    is($hParams->{'ViewAction'}, 'ViewPaymentPayPalPlusResult', "$SubTestName - Test if ViewAction is correct");
    is($hParams->{'Reason'}, 'PayPalPlus_PaymentFailed', "$SubTestName - Test if Reason is cancel");
    is($hParams->{'Name'}, 'PayPalPlus_PaymentFailed', "$SubTestName - Test if Name is cancel");
    is($hParams->{'ObjectID'}, undef, "$SubTestName - Test if ObjectId is right");
    is($hParams->{'ObjectClass'}, undef,"$SubTestName - Test if ObjectClass is Order");
}

sub test_ViewPayPalPlusCancelPayment_UnityShop {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    $self->{'__setup'}->{'IsUnityShop'} = 1;

    my $WasOrderLoaded = 1;
    my $WasDoRequestSuccessful = 1;
    my $ProccessOrderId = 42;
    my $UIShop = $self->_createUIShop($WasOrderLoaded, $WasDoRequestSuccessful, $ProccessOrderId);
    my $Servlet = $self->_buildServlet();

    lives_ok(sub { $UIShop->ViewPayPalPlusCancelPayment($Servlet) }, "$SubTestName - Test if ViewPayPalPlusCancelPayment lives");

    my $LineItemPayment = $self->{'__toCheck'}->{'LineItemPayment'};

    is($LineItemPayment->get('PaymentErrorCode'), 'Failed', "$SubTestName - Test if PaymentErrorCode is Failed");
}

sub test_PaymentBacklinkPayPalPlus {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    $self->{'__setup'}->{'IsUnityShop'} = 0;

    my $WasOrderLoaded = 1;
    my $WasDoRequestSuccessful = 1;
    my $ProccessOrderId = 42;
    my $UIShop = $self->_createUIShop($WasOrderLoaded, $WasDoRequestSuccessful, $ProccessOrderId);
    my $Servlet = $self->_buildServlet();

    lives_ok(sub { $UIShop->PaymentBacklinkPayPalPlus($Servlet) }, "$SubTestName - Test if PaymentBacklinkPayPalPlus lives");
    is(WasCalled($UIShop, 'BlockMonitor'), 1, "$SubTestName - Test if BlockMonitor was called");
    is($Servlet->vars('WasSuccessful'), 1, "$SubTestName - Test if WasSuccessful was set correctly");
    is($Servlet->vars('ViewObjectID'), 'CustomerOrderID', "$SubTestName - Test if ViewObjectID was set correctly"); ## no critic (ProhibitMagicNumbers)

    my $ContextLogger = $UIShop->{'__ToCheck'}->{'_ContextLogger'};
    my $aParams_logContext = $ContextLogger->{'__ToCheck'}->{'_Params_logContext'};
    my $ShopObject = @{$aParams_logContext}[0];
    my $MethodName = @{$aParams_logContext}[1];
    my $FunctionPointer = @{$aParams_logContext}[2];

    ok($ShopObject->isa('DE_EPAGES::Shop::API::Object::Shop'), "$SubTestName - Test if the shop object was given to _logContext");
    is($MethodName, 'DE_EPAGES::PayPalPlus::UI::Shop::PaymentBacklinkPayPalPlus', "$SubTestName - Test if the string was given to _logContext");
    is(ref $FunctionPointer, 'CODE', "$SubTestName - Test if the function was given to _logContext");

    my $Logger = $UIShop->{'__ToCheck'}->{'_Logger'};
    my $Params_logPaymentWithoutPassword = GetParametersFromMockifyCall($Logger, 'logPayment');
    my $Param1 = $Params_logPaymentWithoutPassword->[0];
    my $Param2 = $Params_logPaymentWithoutPassword->[1];
    is($Param1,'Received Params',"$SubTestName - Test log Description");
    my $hExpectedHash = {
          'ServletParams' => {
                              'BaseLineContentLocation' => 'http://domain.de/',
                              'DereferrerToken' => 'token',
                              'PaymentLineItemGUID' => 'id',
                              'PayerID' => 'TestPayerId',
                              'paymentId' => 'paymentid',
                              'QUERY_STRING' => 'importantparams&DereferrerToken=unimportantstuff',
                              'ObjectID' => 'objectid',
                              'ChangeAction' => 'changeaction'
                            }
    };
    is_deeply($Param2, $hExpectedHash,"$SubTestName - Test information that is given to log");
}

sub test_PaymentBacklinkPayPalPlus_FailToProcessOrder {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    $self->{'__setup'}->{'IsUnityShop'} = 0;

    my $WasOrderLoaded = 0;
    my $WasDoRequestSuccessful = 1;
    my $ProccessOrderId = 0;
    my $UIShop = $self->_createUIShop($WasOrderLoaded, $WasDoRequestSuccessful, $ProccessOrderId);
    my $Servlet = $self->_buildServlet();

    lives_ok(sub { $UIShop->PaymentBacklinkPayPalPlus($Servlet); }, "$SubTestName - Test if PaymentBacklinkPayPalPlus lives");
    is(WasCalled($UIShop, 'BlockMonitor'), 1, "$SubTestName - Test if BlockMonitor was called");

    my $ContextLogger = $UIShop->{'__ToCheck'}->{'_ContextLogger'};
    my $aParams_logContext = $ContextLogger->{'__ToCheck'}->{'_Params_logContext'};
    my $ShopObject = @{$aParams_logContext}[0];
    my $MethodName = @{$aParams_logContext}[1];
    my $FunctionPointer = @{$aParams_logContext}[2];

    ok($ShopObject->isa('DE_EPAGES::Shop::API::Object::Shop'), "$SubTestName - Test if the shop object was given to _logContext");
    is($MethodName, 'DE_EPAGES::PayPalPlus::UI::Shop::PaymentBacklinkPayPalPlus', "$SubTestName - Test if the string was given to _logContext");
    is(ref $FunctionPointer, 'CODE', "$SubTestName - Test if the function was given to _logContext");

    my $Form = $self->{'__toCheck'}->{'Form'};
    my $aParamsExecuteFormError = GetParametersFromMockifyCall($Form, 'executeFormError', 0);
    my $hParams = $aParamsExecuteFormError->[0];
    is($hParams->{'ViewAction'}, 'ViewPaymentPayPalPlusResult', "$SubTestName - Test if ViewAction is correct");
    is($hParams->{'Reason'}, 'PayPalPlus_PaymentFailed', "$SubTestName - Test if Reason is cancel");
    is($hParams->{'Name'}, 'PayPalPlus_PaymentFailed', "$SubTestName - Test if Name is cancel");
    is($hParams->{'ObjectID'}, 'CustomerOrderID', "$SubTestName - Test if ObjectId is right");
    is($hParams->{'ObjectClass'}, 'CustomerOrder',"$SubTestName - Test if ObjectClass is Order");
}

sub test_PaymentBacklinkPayPalPlus_FailToProcessOrder_UnityShop {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    $self->{'__setup'}->{'IsUnityShop'} = 1;

    my $WasOrderLoaded = 0;
    my $WasDoRequestSuccessful = 1;
    my $ProccessOrderId = 0;
    my $UIShop = $self->_createUIShop($WasOrderLoaded, $WasDoRequestSuccessful, $ProccessOrderId);
    my $Servlet = $self->_buildServlet();

    lives_ok(sub { $UIShop->PaymentBacklinkPayPalPlus($Servlet); }, "$SubTestName - Test if PaymentBacklinkPayPalPlus lives");
    is(WasCalled($UIShop, 'BlockMonitor'), 1, "$SubTestName - Test if BlockMonitor was called");

    my $ContextLogger = $UIShop->{'__ToCheck'}->{'_ContextLogger'};
    my $aParams_logContext = $ContextLogger->{'__ToCheck'}->{'_Params_logContext'};
    my $ShopObject = @{$aParams_logContext}[0];
    my $MethodName = @{$aParams_logContext}[1];
    my $FunctionPointer = @{$aParams_logContext}[2];

    ok($ShopObject->isa('DE_EPAGES::Shop::API::Object::Shop'), "$SubTestName - Test if the shop object was given to _logContext");
    is($MethodName, 'DE_EPAGES::PayPalPlus::UI::Shop::PaymentBacklinkPayPalPlus', "$SubTestName - Test if the string was given to _logContext");
    is(ref $FunctionPointer, 'CODE', "$SubTestName - Test if the function was given to _logContext");

    my $LineItemPayment = $self->{'__toCheck'}->{'LineItemPayment'};
    is($LineItemPayment->get('PaymentErrorCode'), 'Failed', "$SubTestName - Test if PaymentErrorCode is Failed");
}

#---------------------------------------------------------------------------
#  helpers
#---------------------------------------------------------------------------

sub _createPaymentLineItemService {
    my $self = shift;
    my ($CanLoadByGuid, $CustomerOrder) = @_;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::PayPalPlus::API::PaymentLineItemService', []);
    $Mockify->addMockWithReturnValueAndParameterCheck('canLoadByGuid', $CanLoadByGuid, ['string']);
    $Mockify->addMockWithReturnValue('getBasketOrOrder', $CustomerOrder);
    $Mockify->addMockWithReturnValue('getErrorMessage', 'ErrorMessage');

    return $Mockify->getMockObject();
}

sub _createUIShop {
    my $self = shift;
    my ($CanLoadByGuid, $WasDoRequestSuccessful, $ProccessOrder) = @_;

    my $CustomerOrder = $self->_createCustomerOrder();

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::PayPalPlus::UI::Shop', [Servlet => $self->_createServlet()]);
    $Mockify->addMockWithReturnValue('_createOrderService', $self->_createOrderService($ProccessOrder, $CustomerOrder));
    $Mockify->addMockWithReturnValue('_createPaymentLineItemService', $self->_createPaymentLineItemService($CanLoadByGuid, $CustomerOrder));
    $Mockify->addMockWithReturnValueAndParameterCheck('_ValidateDereferrerToken', undef, [{'string'=>'token'}, {'string'=>'ShopAlias&objectid&id'}]);
    $Mockify->addMock('BlockMonitor', sub {});
    $Mockify->addMockWithReturnValueAndParameterCheck('_LogError', undef, ['string', 'string']);

    my $LogContext = $self->_createContextLogger();
    $Mockify->addMockWithReturnValue('_createLogContext', $LogContext);
    my $Logger = $self->_createLogger();
    $Mockify->addMockWithReturnValue('_createLogger', $Logger);

    my $UI_Shop = $Mockify->getMockObject();
    $UI_Shop->{'__ToCheck'}->{'_ContextLogger'} = $LogContext;
    $UI_Shop->{'__ToCheck'}->{'_Logger'} = $Logger;
    return $UI_Shop;
}
#----------------------------------------------------------------------------------------
sub _createLogger {
    my $self = shift;
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new( 'DE_EPAGES::PayPalPlus::API::Logger', [] );

    $Mockify->addMockWithReturnValueAndParameterCheck('logPayment', undef, ['string', 'hashref']);

    return $Mockify->getMockObject();
}
sub _createOrderService {
    my $self = shift;
    my ($ProccessOrder, $CustomerOrder) = @_;
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new( 'DE_EPAGES::PayPalPlus::API::OrderService', [] );

    $Mockify->addMockWithReturnValueAndParameterCheck('proccessOrder', $ProccessOrder, ['hashref']);
    $Mockify->addMockWithReturnValue('getErrorMessage', 'ErrorAtProccessingOrder');
    $Mockify->addMockWithReturnValue('getBasketOrOrder', $CustomerOrder);

    return $Mockify->getMockObject();
}
sub _createContextLogger {
    my $self = shift;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new( 'DE_EPAGES::PayPalPlus::API::ContextLogger', [] );
    $Mockify->addMock('logContext',
        sub {
            my $InternalSelf = shift;
            $InternalSelf->{'__ToCheck'}->{'_Params_logContext'} = \@_;
            my $FunctionPointer = $InternalSelf->{'__ToCheck'}->{'_Params_logContext'}[2];
            return $FunctionPointer->();
        }
    );
    return $Mockify->getMockObject();
}


sub _buildServlet {
    my $self = shift;
    return $self->_createServlet('TestPayerId', 'paymentid');
}

sub _createServlet {
    my $self = shift;
    my ($PayerId, $PaymentId) = @_;

    my $Shop = $self->_createShop();
    my $MockifyServlet = DE_EPAGES::Test::Mock::MockifyServlet->new($Shop);

    $MockifyServlet->addVars(
        {
            'OrderID' => undef,
            'WasSuccessful' => undef,
            'ViewObjectID' => undef,
            'ViewAction' => undef,
            'OrderID' => undef,
            'ErrorCode' => undef,
            'ErrorMessage' => undef,
        }
    );

    $MockifyServlet->addMockWithReturnValueAndParameterCheck('locale', undef, [
        {'int' => 1}
    ]);
    $MockifyServlet->addMockWithReturnValueAndParameterCheck('languageID', undef, [
        {'int' => 1}
    ]);
    $MockifyServlet->addMockWithReturnValueAndParameterCheck('currencyID', undef, [
        {'int' => 1}
    ]);
    $MockifyServlet->addMockWithReturnValueAndParameterCheck('registerSession', undef, [
        {'string' => 'Tom'},
        {'int' => 0},
    ]);
    $MockifyServlet->addMockWithReturnValue('form', $self->_createForm());
    $MockifyServlet->addMockWithReturnValue('requestHeaders', {});
    $MockifyServlet->addMockWithReturnValueAndParameterCheck('redirect', undef, [{'string'=> 'http://unity.url'}]);
    $MockifyServlet->addParams( {
        'ChangeAction' => 'changeaction',
        'ObjectID' => 'objectid',
        'PaymentLineItemGUID' => 'id',
        'BaseLineContentLocation' => 'http://domain.de/',
        'QUERY_STRING' => 'importantparams&DereferrerToken=unimportantstuff',
        'DereferrerToken' => 'token',
        'PayerID' => $PayerId,
        'paymentId' => $PaymentId,
    });

    return $MockifyServlet->getMockObject();
}

sub _createForm {
    my $self = shift;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Presentation::API::Form', []);
    $Mockify->addMockWithReturnValueAndParameterCheck('executeFormError', undef, ['hashref']);

    my $Form = $Mockify->getMockObject();
    $self->{'__toCheck'}->{'Form'} = $Form;
    return $Form;
}

sub _createShop {
    my $self = shift;
    my $hAttributes = {
        'Alias' => 'ShopAlias',
        'ID' => 'ShopID',
        'IsUnityRegistered' => $self->{'__setup'}->{'IsUnityShop'}
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', $hAttributes);
}

sub _createCustomerOrder {
    my $self = shift;

    my $hAttributes = {
        'Shop' => $self->_createShop(),
        'ID' => 'CustomerOrderID',
        'LineItemContainer' => $self->_createLineItemContainer(),
        'WebUrlUnitySSL' => 'http://unity.url',
        'User' => 'Tom'
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Order::API::Object::CustomerOrder', $hAttributes);
    $Mockify->addMockWithReturnValue('id', 'CustomerOrderID');
    $Mockify->addMockWithReturnValue('class', $self->_createClass('CustomerOrder'));

    return $Mockify->getMockObject();
}

sub _createLineItemContainer {
    my $self = shift;

    my $hAttributes = {
        'Payment' => $self->_createLineItemPayment()
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Order::API::Object::LineItemContainer',
        $hAttributes
    );

    return $Mockify->getMockObject();
}

sub _createLineItemPayment {
    my $self = shift;

    my $hAttributes = {
        'Alias' => 'MyShop',
        'PaymentErrorCode' => undef
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Order::API::Object::LineItemPayment',
        $hAttributes
    );

    my $LineItemPayment = $Mockify->getMockObject();
    $self->{'__toCheck'}->{'LineItemPayment'} = $LineItemPayment;
    return $LineItemPayment;
}

sub _createClass {
    my $self = shift;
    my ($Alias) = @_;
    my $hAttributes = {
        'Alias' => $Alias,
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Object::API::Object::Class', $hAttributes);
}

__PACKAGE__->runScript();