package DE_EPAGES::Shop::t::UpdateUnity;

use base DE_EPAGES::Core::API::Test;
use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use Test::Exception;
use Test::More;
use strict;

sub test {
    my $self = shift;
    $self->_updateUnityConnected();
    $self->_updateUnityDisconnected();
    done_testing();
}

sub _updateUnityConnected {
    my $self        = shift;
    my $SubTestName = (caller(0))[3];
    my $Response;
    my $hAttributes = {
        'Alias'                       => 'MyTestShop',
        'ClosedByProvider'            => 1,
        'ClosedByMerchant'            => 0,
        'ClosedByProviderTemporarily' => 1,
        'ProtocolAndServer'           => 'http://epages.my-server.com',
        'WebPath'                     => '/WebRoot/Store/Shops/MyTestShop'
    };
    my $MockifyShop = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Shop::API::Object::Shop', $hAttributes);
    $MockifyShop->addMock(
        'sendUnityApiPatchCall',
        sub {
            my $self = shift;
            my ($hContent) = @_;

            $Response = $hContent;
        });
    my $Shop = $MockifyShop->getMockObject();

    $Response = undef;
    $Shop->updateUnityShop();
    is($Response, undef, "$SubTestName without content --> does nothing");
    $Response = undef;
    $Shop->updateUnityShop({'ShopAlias' => 'Test'});
    is($Response, undef,
        "$SubTestName with irrelevant content (Alias) --> does nothing");

    $Response = undef;
    $Shop->updateUnityShop({'ClosedByProvider' => 1});
    is($Response->{'isClosed'} , "true",
        "$SubTestName with ClosedByProvider = 1 --> closed Shop");
    $Response = undef;
    $Shop->updateUnityShop({'ClosedByMerchant' => 0});
    is($Response->{'isClosed'}, "false",
        "$SubTestName with ClosedByMerchant = 0 --> opened Shop");
    $Response = undef;
    $Shop->updateUnityShop({
        'ClosedByProviderTemporarily' => 1,
        'ClosedByMerchant'            => 0
        });
    is($Response->{'isClosed'}, "true",
        "$SubTestName with ClosedByProviderTemporarily = 1 and ClosedByMerchant = 0 --> closed Shop");
    $Response = undef;
    $Shop->updateUnityShop($Shop->{'Attributes'});
    is($Response->{'isClosed'}, "true",
        "$SubTestName with all shop attributes --> closed Shop");

    $Response = undef;
    $Shop->updateUnityShop({'Name' => "Spotlight"});
    is($Response->{'title'}, "Spotlight",
        "$SubTestName with Name = Spotlight");

    $Response = undef;
    $Shop->updateUnityShop({'IsTrialShop' => 1});
    is($Response, undef,
        "$SubTestName with IsTrialShop --> no response");
    $Response = undef;
    $Shop->updateUnityShop({'DomainName' => "www.epages.com"});
    is($Response->{'domain'}, "www.epages.com",
        "$SubTestName with DomainName = www.epages.com");
    $Response = undef;
    $Shop->updateUnityShop({'Logo' => 'logo.jpg'});
    is($Response->{'logoUrl'}, 'http://epages.my-server.com/WebRoot/Store/Shops/MyTestShop/logo.jpg',
        "$SubTestName with Logo = logo.jpg");
    $Response = undef;
    $Shop->updateUnityShop({'Logo' => undef});
    is($Response->{'logoUrl'}, undef,
        "$SubTestName delete logo");
    $Response = undef;
    $Shop->updateUnityShop({'GoogleAnalyticsWebPropertyID' => '08154711'});
    is($Response->{'userSettings'}->{'googleAnalytics'}->{'trackingId'}, "08154711",
        "$SubTestName update google analytics id");
}

sub _updateUnityDisconnected {
    my $self        = shift;
    my $SubTestName = (caller(0))[3];

    my $hAttributes = {
        'Alias'                       => 'MyTestShop'
    };
    my $MockifyShop = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Shop::API::Object::Shop', $hAttributes);

    $MockifyShop->addMock(
        'sendUnityApiPatchCall',
        sub {
            my $self = shift;
            my ($hContent) = @_;

            Error('UNITY_API_CALL_ERROR');
        });
    my $Shop = $MockifyShop->getMockObject();

    throws_ok( sub {$Shop->updateUnityShop({'ClosedByMerchant' => 0})}, qr/UNITY_API_CALL_ERROR/,
        "$SubTestName with ClosedByMerchant = 0 --> got error response");
    lives_ok( sub {$Shop->updateUnityShop({'ShopAlias' => 'Test'})},
        "$SubTestName with irrelevant content (Alias) --> no response");
}

__PACKAGE__->runScript();
