#========================================================================================
# package      DE_EPAGES::Shop::t::unit::ShopUtil
# base         DE_EPAGES::Test::API::UnitTest::Test
#----------------------------------------------------------------------------------------
# description  unit test for ShopUtil
#========================================================================================
package DE_EPAGES::Shop::t::unit::ShopUtil;
use base DE_EPAGES::Test::API::UnitTest::Test;
use DE_EPAGES::Shop::API::ShopUtil;
use Test::More;

use strict;

sub test {
    my $self = shift;

    $self->test_addMandatoryAddressFields();
    $self->test_addMandatoryBillingAddressFields();
    $self->test_addMandatoryBillingAddressFields_emptyArray();
    $self->test_addMandatoryBillingAddressFields_noArray();
    $self->test_addMandatoryBillingAddressFields_alreadyExistingField();
    $self->test_addMandatoryShippingAddressFields();
    $self->test_addMandatoryShippingAddressFields_emptyArray();
    $self->test_addMandatoryShippingAddressFields_noArray();
    $self->test_addMandatoryShippingAddressFields_alreadyExistingField();
    done_testing();
}
#----------------------------------------------------------------------------------------
sub test_addMandatoryAddressFields {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->mockFactory()->buildShop({
            'Alias' => 'TestShop',
            'BillingAddressSetMandatoryFields' => 'field1',
            'BillingAddressSetVisibleFields' => 'field1,field2',
            'ShippingAddressSetMandatoryFields' => 'field2',
            'ShippingAddressSetVisibleFields' => 'field2,field3'
    })->getMockObject();
    my $SUT = DE_EPAGES::Shop::API::ShopUtil->new($Shop);

    $SUT->addMandatoryAddressFields(['billingfield'], ['shippingfield']);

    is($Shop->get('BillingAddressSetMandatoryFields'), 'field1,billingfield', "$SubTestName - check if billingfield is appended correctly");
    is($Shop->get('BillingAddressSetVisibleFields'), 'field1,field2,billingfield', "$SubTestName - check if billingfield is appended correctly");
    is($Shop->get('ShippingAddressSetMandatoryFields'), 'field2,shippingfield', "$SubTestName - check if shippingfield is appended correctly");
    is($Shop->get('ShippingAddressSetVisibleFields'), 'field2,field3,shippingfield', "$SubTestName - check if shippingfield is appended correctly");
}

#----------------------------------------------------------------------------------------
sub test_addMandatoryBillingAddressFields {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->mockFactory()->buildShop({
            'Alias' => 'TestShop',
            'BillingAddressSetMandatoryFields' => 'field1',
            'BillingAddressSetVisibleFields' => 'field1,field2',
            'ShippingAddressSetMandatoryFields' => 'field2',
            'ShippingAddressSetVisibleFields' => 'field2,field3'
    })->getMockObject();
    my $SUT = DE_EPAGES::Shop::API::ShopUtil->new($Shop);

    $SUT->addMandatoryBillingAddressFields(['billingfield']);

    is($Shop->get('BillingAddressSetMandatoryFields'), 'field1,billingfield', "$SubTestName - check if billingfield is appended correctly");
    is($Shop->get('BillingAddressSetVisibleFields'), 'field1,field2,billingfield', "$SubTestName - check if billingfield is appended correctly");
}

#----------------------------------------------------------------------------------------
sub test_addMandatoryBillingAddressFields_emptyArray {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->mockFactory()->buildShop({
            'Alias' => 'TestShop',
            'BillingAddressSetMandatoryFields' => 'field1',
            'BillingAddressSetVisibleFields' => 'field1,field2',
            'ShippingAddressSetMandatoryFields' => 'field2',
            'ShippingAddressSetVisibleFields' => 'field2,field3'
    })->getMockObject();
    my $SUT = DE_EPAGES::Shop::API::ShopUtil->new($Shop);

    $SUT->addMandatoryBillingAddressFields([]);

    is($Shop->get('BillingAddressSetMandatoryFields'), 'field1', "$SubTestName - check if nothing is appended");
    is($Shop->get('BillingAddressSetVisibleFields'), 'field1,field2', "$SubTestName - check if nothing is appended");
}

#----------------------------------------------------------------------------------------
sub test_addMandatoryBillingAddressFields_noArray {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->mockFactory()->buildShop({
            'Alias' => 'TestShop',
            'BillingAddressSetMandatoryFields' => 'field1',
            'BillingAddressSetVisibleFields' => 'field1,field2',
            'ShippingAddressSetMandatoryFields' => 'field2',
            'ShippingAddressSetVisibleFields' => 'field2,field3'
    })->getMockObject();
    my $SUT = DE_EPAGES::Shop::API::ShopUtil->new($Shop);

    $SUT->addMandatoryBillingAddressFields();

    is($Shop->get('BillingAddressSetMandatoryFields'), 'field1', "$SubTestName - check if nothing is appended");
    is($Shop->get('BillingAddressSetVisibleFields'), 'field1,field2', "$SubTestName - check if nothing is appended");
}

#----------------------------------------------------------------------------------------
sub test_addMandatoryBillingAddressFields_alreadyExistingField {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->mockFactory()->buildShop({
            'Alias' => 'TestShop',
            'BillingAddressSetMandatoryFields' => 'field1',
            'BillingAddressSetVisibleFields' => 'field1,field2',
            'ShippingAddressSetMandatoryFields' => 'field2',
            'ShippingAddressSetVisibleFields' => 'field2,field3'
    })->getMockObject();
    my $SUT = DE_EPAGES::Shop::API::ShopUtil->new($Shop);

    $SUT->addMandatoryBillingAddressFields(['field1']);

    is($Shop->get('BillingAddressSetMandatoryFields'), 'field1', "$SubTestName - check if nothing is appended");
    is($Shop->get('BillingAddressSetVisibleFields'), 'field1,field2', "$SubTestName - check if nothing is appended");
}

#----------------------------------------------------------------------------------------
sub test_addMandatoryShippingAddressFields {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->mockFactory()->buildShop({
            'Alias' => 'TestShop',
            'BillingAddressSetMandatoryFields' => 'field1',
            'BillingAddressSetVisibleFields' => 'field1,field2',
            'ShippingAddressSetMandatoryFields' => 'field2',
            'ShippingAddressSetVisibleFields' => 'field2,field3'
    })->getMockObject();
    my $SUT = DE_EPAGES::Shop::API::ShopUtil->new($Shop);

    $SUT->addMandatoryShippingAddressFields(['shippingfield']);

    is($Shop->get('ShippingAddressSetMandatoryFields'), 'field2,shippingfield', "$SubTestName - check if billingfield is appended correctly");
    is($Shop->get('ShippingAddressSetVisibleFields'), 'field2,field3,shippingfield', "$SubTestName - check if billingfield is appended correctly");
}

#----------------------------------------------------------------------------------------
sub test_addMandatoryShippingAddressFields_emptyArray {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->mockFactory()->buildShop({
            'Alias' => 'TestShop',
            'BillingAddressSetMandatoryFields' => 'field1',
            'BillingAddressSetVisibleFields' => 'field1,field2',
            'ShippingAddressSetMandatoryFields' => 'field2',
            'ShippingAddressSetVisibleFields' => 'field2,field3'
    })->getMockObject();
    my $SUT = DE_EPAGES::Shop::API::ShopUtil->new($Shop);

    $SUT->addMandatoryShippingAddressFields([]);

    is($Shop->get('ShippingAddressSetMandatoryFields'), 'field2', "$SubTestName - check if nothing is appended");
    is($Shop->get('ShippingAddressSetVisibleFields'), 'field2,field3', "$SubTestName - check if nothing is appended");
}

#----------------------------------------------------------------------------------------
sub test_addMandatoryShippingAddressFields_noArray {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->mockFactory()->buildShop({
            'Alias' => 'TestShop',
            'BillingAddressSetMandatoryFields' => 'field1',
            'BillingAddressSetVisibleFields' => 'field1,field2',
            'ShippingAddressSetMandatoryFields' => 'field2',
            'ShippingAddressSetVisibleFields' => 'field2,field3'
    })->getMockObject();
    my $SUT = DE_EPAGES::Shop::API::ShopUtil->new($Shop);

    $SUT->addMandatoryShippingAddressFields();

    is($Shop->get('ShippingAddressSetMandatoryFields'), 'field2', "$SubTestName - check if nothing is appended");
    is($Shop->get('ShippingAddressSetVisibleFields'), 'field2,field3', "$SubTestName - check if nothing is appended");
}

#----------------------------------------------------------------------------------------
sub test_addMandatoryShippingAddressFields_alreadyExistingField {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->mockFactory()->buildShop({
            'Alias' => 'TestShop',
            'BillingAddressSetMandatoryFields' => 'field1',
            'BillingAddressSetVisibleFields' => 'field1,field2',
            'ShippingAddressSetMandatoryFields' => 'field2',
            'ShippingAddressSetVisibleFields' => 'field2,field3'
    })->getMockObject();
    my $SUT = DE_EPAGES::Shop::API::ShopUtil->new($Shop);

    $SUT->addMandatoryShippingAddressFields(['field2']);

    is($Shop->get('ShippingAddressSetMandatoryFields'), 'field2', "$SubTestName - check if nothing is appended");
    is($Shop->get('ShippingAddressSetVisibleFields'), 'field2,field3', "$SubTestName - check if nothing is appended");
}

__PACKAGE__->runScript();
